import { useEffect, useRef, useState } from 'react';
import { useTranslation } from 'react-i18next';
import classNames from 'classnames';
import { select as d3select, zoom as d3zoom, geoIdentity, geoPath, rgb, scaleLinear } from 'd3';
import { feature } from 'topojson-client';
// eslint-disable-next-line import/no-unresolved

import KeyValueTooltip from '../KeyValueTooltip/KeyValueTooltip.component';
import styles from './GeoChart.module.scss';

// Rename ugly d3 types
import { Icon } from "@talend/react-components";
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
// Don't worry, chart will fit it's container
const width = 250;
const height = 200;

// Reuse collator for better performance
const COLLATOR = new Intl.Collator(undefined, {
  usage: 'search',
  sensitivity: 'base',
  ignorePunctuation: true
});
function findEntry(data, properties, chartConfig) {
  // Match configured fields
  return data.find(entry => chartConfig.valueProperties.some(property => COLLATOR.compare(entry.key, `${properties[property]}`) === 0));
}
function renderFeature(path, colorScale, setTooltip, onSelection, entry, label) {
  path.attr('class', classNames(styles['geo-chart__feature'], {
    [styles['geo-chart__feature--disabled']]: !entry
  }));
  if (entry) {
    var _colorScale;
    // if we have both label and entry key: "label (key): value", otherwise "label: value" or "key: value"
    const tooltipLabel = `${label || entry.key}${label && entry.key !== label ? ` (${entry.key})` : ''}`;
    const setTooltipFromD3Event = d3Event => {
      setTooltip({
        entry: {
          key: tooltipLabel,
          value: `${entry.value}`
        },
        position: {
          x: d3Event.x,
          y: d3Event.y
        }
      });
    };
    path.attr('data-key', entry.key).attr('data-value', entry.value).style('fill', ((_colorScale = colorScale(+entry.value)) === null || _colorScale === void 0 ? void 0 : _colorScale.toString()) || '').on('click', () => onSelection(entry.key)).on('mouseover', d3Event => setTooltipFromD3Event(d3Event)).on('mousemove', d3Event => d3Event && setTooltipFromD3Event(d3Event)).on('mouseout', () => setTooltip(null));
  }
}
function createSvg(parent) {
  return d3select(parent).append('svg').attr('viewBox', `0 0 ${width} ${height}`);
}
function clearChart(container) {
  d3select(container).selectAll('svg').remove();
}
function parseCssVarColor(cssVarExpression) {
  try {
    return cssVarExpression.split(',').slice(1).join(',').trim().slice(0, -1);
  } catch (e) {
    return 'hsl(0, 0%, 0%)';
  }
}
function getScale(data) {
  const values = data.map(entry => entry.value);
  return scaleLinear().domain([Math.min(...values), Math.max(...values)]).range([rgb(parseCssVarColor(styles.scaleMinColor)), rgb(parseCssVarColor(styles.scaleMaxColor))]);
}
function getGeoPath(featureCollection) {
  // we use the same projection for all maps: custom projections have to be applied to the topojson file
  const projection = geoIdentity().scale(1).fitSize([width, height], featureCollection);
  return geoPath().projection(projection);
}
function getFeatureCollection(chartConfig) {
  // Use configured layer or first available
  const topology = chartConfig.topology;
  const layer = chartConfig.layer || Object.keys(topology.objects)[0];
  // convert topojson to geojson
  return feature(topology, topology.objects[layer]);
}
function registerZoom(svg, container, zoomInButtonRef, zoomOutButtonRef) {
  const zoom = d3zoom().scaleExtent([1, 100]).on('zoom', d3Event => {
    const currentTransform = d3Event.transform;
    container.attr('transform', currentTransform);
  });
  d3select(zoomInButtonRef).on('click', () => {
    zoom.scaleBy(svg.transition().duration(250), 2);
  });
  d3select(zoomOutButtonRef).on('click', () => {
    zoom.scaleBy(svg.transition().duration(250), 1 / 2);
  });
  svg.call(zoom);
}
function GeoChart({
  data,
  columnName,
  onSelection,
  chartConfig
}) {
  var _tooltipRef$current, _tooltipRef$current2;
  const containerRef = useRef(null);
  const tooltipRef = useRef(null);
  const zoomInButtonRef = useRef(null);
  const zoomOutButtonRef = useRef(null);
  const {
    t
  } = useTranslation();
  const [tooltip, setTooltip] = useState();
  useEffect(() => {
    if (chartConfig && containerRef.current && zoomInButtonRef.current && zoomOutButtonRef.current) {
      clearChart(containerRef.current);
      const colorScale = getScale(data);
      const featureCollection = getFeatureCollection(chartConfig);
      const svg = createSvg(containerRef.current);
      const container = svg.append('g').attr('data-test', 'preparation.chart.geochart');
      container.selectAll('path').data(featureCollection.features).enter().append('path').each(function render(d) {
        var _d$properties;
        const matchedEntry = d.properties ? findEntry(data, d.properties, chartConfig) : undefined;
        const label = chartConfig.labelProperty && ((_d$properties = d.properties) === null || _d$properties === void 0 ? void 0 : _d$properties[chartConfig.labelProperty]);
        renderFeature(d3select(this), colorScale, setTooltip, onSelection, matchedEntry, label);
      }).attr('d', getGeoPath(featureCollection));
      registerZoom(svg, container, zoomInButtonRef.current, zoomOutButtonRef.current);
    }
  }, [data, chartConfig, onSelection, setTooltip]);
  return /*#__PURE__*/_jsxs("div", {
    className: styles['geo-chart'],
    children: [/*#__PURE__*/_jsx("h1", {
      className: styles['geo-chart__title'],
      "data-test": "preparation.chart.title",
      children: t('GEO_DISTRIBUTION', {
        column_name: columnName,
        defaultValue: '{{column_name}} distribution'
      })
    }), /*#__PURE__*/_jsxs("div", {
      className: styles['geo-chart__zoom'],
      children: [/*#__PURE__*/_jsx("button", {
        ref: zoomInButtonRef,
        title: t('ZOOM_IN', {
          defaultValue: 'Zoom in'
        }),
        className: styles['geo-chart__zoom-button'],
        "data-feature": "preparation.chart.zoom-in",
        children: /*#__PURE__*/_jsx(Icon, {
          name: "talend-plus-circle",
          className: styles['geo-chart__zoom-icon']
        })
      }), /*#__PURE__*/_jsx("button", {
        ref: zoomOutButtonRef,
        title: t('ZOOM_OUT', {
          defaultValue: 'Zoom out'
        }),
        className: styles['geo-chart__zoom-button'],
        "data-feature": "preparation.chart.zoom-out",
        children: /*#__PURE__*/_jsx(Icon, {
          name: "talend-minus-circle",
          className: styles['geo-chart__zoom-icon']
        })
      })]
    }), /*#__PURE__*/_jsx("div", {
      ref: containerRef
    }), tooltip && /*#__PURE__*/_jsx("div", {
      ref: tooltipRef,
      style: {
        position: 'fixed',
        top: `${tooltip.position.y - (((_tooltipRef$current = tooltipRef.current) === null || _tooltipRef$current === void 0 ? void 0 : _tooltipRef$current.offsetHeight) || 0)}px`,
        left: `${tooltip.position.x - (((_tooltipRef$current2 = tooltipRef.current) === null || _tooltipRef$current2 === void 0 ? void 0 : _tooltipRef$current2.offsetWidth) || 0) / 2}px`,
        pointerEvents: 'none'
      },
      children: /*#__PURE__*/_jsx(KeyValueTooltip, {
        entries: [tooltip.entry]
      })
    })]
  });
}
export default GeoChart;
//# sourceMappingURL=GeoChart.component.js.map