import { useState } from 'react';
import { LineChart as RLineChart, Line, CartesianGrid, ResponsiveContainer, Tooltip, XAxis, YAxis, Legend } from 'recharts';
import tokens from '@talend/design-tokens';
import { CustomTooltip } from './LineChartTooltip.component';
import { CustomLegend } from './LineChartLegend.component';
import style from './LineChart.module.scss';
import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
function LineChart({
  data,
  lines,
  chartOptions,
  hasLineSelection,
  initialSelectedLines = [],
  onLineClicked = () => {},
  onLineHovered = () => {},
  onLegendItemClicked = () => {},
  onLegendItemHovered = () => {}
}) {
  const {
    width,
    height,
    margin,
    showGridLines,
    xAxisOptions,
    leftYAxisOptions,
    rightYAxisOptions,
    legend,
    tooltip
  } = chartOptions;
  const [activeLine, setActiveLine] = useState(null);
  const [selectedLines, setSelectedLines] = useState(initialSelectedLines);
  const hasOnlyOneValue = (data === null || data === void 0 ? void 0 : data.length) === 1;
  const getLineStyleFromStatus = (status, key) => {
    const defaultDotSize = hasOnlyOneValue ? 2 : 0;
    const defaultDotStrokeWidth = hasOnlyOneValue ? 4 : 0;
    const styleByStatus = {
      light: {
        strokeWidth: 1,
        strokeOpacity: 1,
        activeDot: {
          r: 3,
          strokeWidth: 0
        }
      },
      active: {
        strokeWidth: 2,
        strokeOpacity: 1,
        activeDot: {
          r: 5,
          strokeWidth: 0
        }
      },
      inactive: {
        strokeWidth: 2,
        strokeOpacity: 0.25,
        activeDot: {
          r: 0,
          strokeWidth: 0
        }
      },
      highlighted: {
        strokeWidth: 3,
        strokeOpacity: 1,
        activeDot: {
          r: 6,
          strokeWidth: 0
        }
      }
    };
    if (hasLineSelection && selectedLines.length > 0) {
      return {
        dot: {
          r: defaultDotSize,
          strokeWidth: defaultDotStrokeWidth
        },
        ...styleByStatus[status],
        strokeOpacity: selectedLines.includes(key) ? 1 : 0.25
      };
    }
    if (activeLine !== null) {
      return {
        dot: {
          r: defaultDotSize,
          strokeWidth: defaultDotStrokeWidth
        },
        ...styleByStatus[status],
        strokeOpacity: activeLine === key ? 1 : 0.25
      };
    }
    return {
      dot: {
        r: defaultDotSize,
        strokeWidth: defaultDotStrokeWidth
      },
      ...styleByStatus[status]
    };
  };
  const onLegendClicked = key => {
    if (hasLineSelection) {
      const isSelected = selectedLines.includes(key);
      const newSelectedLines = isSelected ? selectedLines.filter(lineKey => lineKey !== key) : [...selectedLines, key];
      setSelectedLines(newSelectedLines.length === lines.length ? [] : newSelectedLines);
    }
    onLegendItemClicked(key);
  };
  const onLegendHovered = key => {
    if (onLegendItemHovered) {
      onLegendItemHovered(key);
    }
    if (key === '') {
      setActiveLine(null);
    } else {
      setActiveLine(key);
    }
  };
  return /*#__PURE__*/_jsx(_Fragment, {
    children: /*#__PURE__*/_jsx(ResponsiveContainer, {
      width: width || '100%',
      height: height || '100%',
      debounce: 1,
      children: /*#__PURE__*/_jsxs(RLineChart, {
        data: data,
        margin: margin || {},
        className: style.container,
        children: [!!showGridLines && /*#__PURE__*/_jsx(CartesianGrid, {
          stroke: tokens.coralColorNeutralBackgroundMedium,
          strokeDasharray: "2",
          vertical: false
        }), /*#__PURE__*/_jsx(XAxis, {
          dataKey: "xLabel",
          interval: xAxisOptions === null || xAxisOptions === void 0 ? void 0 : xAxisOptions.interval,
          dx: xAxisOptions === null || xAxisOptions === void 0 ? void 0 : xAxisOptions.horizontalOffset,
          dy: xAxisOptions === null || xAxisOptions === void 0 ? void 0 : xAxisOptions.verticalOffset,
          allowDataOverflow: xAxisOptions === null || xAxisOptions === void 0 ? void 0 : xAxisOptions.clipDomain,
          ticks: xAxisOptions === null || xAxisOptions === void 0 ? void 0 : xAxisOptions.manualTicks,
          tickLine: false,
          tickFormatter: xAxisOptions === null || xAxisOptions === void 0 ? void 0 : xAxisOptions.formatter,
          domain: xAxisOptions === null || xAxisOptions === void 0 ? void 0 : xAxisOptions.domain,
          type: xAxisOptions === null || xAxisOptions === void 0 ? void 0 : xAxisOptions.type
        }), /*#__PURE__*/_jsx(YAxis, {
          yAxisId: "left",
          type: leftYAxisOptions === null || leftYAxisOptions === void 0 ? void 0 : leftYAxisOptions.type,
          domain: leftYAxisOptions === null || leftYAxisOptions === void 0 ? void 0 : leftYAxisOptions.domain,
          unit: leftYAxisOptions !== null && leftYAxisOptions !== void 0 && leftYAxisOptions.hideUnitInAxis ? '' : leftYAxisOptions === null || leftYAxisOptions === void 0 ? void 0 : leftYAxisOptions.unit,
          interval: leftYAxisOptions !== null && leftYAxisOptions !== void 0 && leftYAxisOptions.manualTicks ? undefined : 'preserveEnd',
          dx: leftYAxisOptions === null || leftYAxisOptions === void 0 ? void 0 : leftYAxisOptions.horizontalOffset,
          dy: leftYAxisOptions === null || leftYAxisOptions === void 0 ? void 0 : leftYAxisOptions.verticalOffset,
          minTickGap: 2,
          tickCount: leftYAxisOptions !== null && leftYAxisOptions !== void 0 && leftYAxisOptions.manualTicks ? undefined : 6,
          ticks: leftYAxisOptions === null || leftYAxisOptions === void 0 ? void 0 : leftYAxisOptions.manualTicks,
          tickLine: !!(leftYAxisOptions !== null && leftYAxisOptions !== void 0 && leftYAxisOptions.tickLine),
          tickFormatter: leftYAxisOptions === null || leftYAxisOptions === void 0 ? void 0 : leftYAxisOptions.formatter
        }), /*#__PURE__*/_jsx(YAxis, {
          hide: (rightYAxisOptions === null || rightYAxisOptions === void 0 ? void 0 : rightYAxisOptions.hide) !== false,
          yAxisId: "right",
          orientation: "right",
          type: rightYAxisOptions === null || rightYAxisOptions === void 0 ? void 0 : rightYAxisOptions.type,
          domain: rightYAxisOptions === null || rightYAxisOptions === void 0 ? void 0 : rightYAxisOptions.domain,
          unit: rightYAxisOptions !== null && rightYAxisOptions !== void 0 && rightYAxisOptions.hideUnitInAxis ? '' : rightYAxisOptions === null || rightYAxisOptions === void 0 ? void 0 : rightYAxisOptions.unit,
          interval: rightYAxisOptions !== null && rightYAxisOptions !== void 0 && rightYAxisOptions.manualTicks ? undefined : 'preserveEnd',
          dx: rightYAxisOptions === null || rightYAxisOptions === void 0 ? void 0 : rightYAxisOptions.horizontalOffset,
          dy: rightYAxisOptions === null || rightYAxisOptions === void 0 ? void 0 : rightYAxisOptions.verticalOffset,
          minTickGap: 2,
          tickCount: rightYAxisOptions !== null && rightYAxisOptions !== void 0 && rightYAxisOptions.manualTicks ? undefined : 6,
          ticks: rightYAxisOptions === null || rightYAxisOptions === void 0 ? void 0 : rightYAxisOptions.manualTicks,
          tickLine: !!(rightYAxisOptions !== null && rightYAxisOptions !== void 0 && rightYAxisOptions.tickLine),
          tickFormatter: rightYAxisOptions === null || rightYAxisOptions === void 0 ? void 0 : rightYAxisOptions.formatter
        }), !(tooltip !== null && tooltip !== void 0 && tooltip.hide) && /*#__PURE__*/_jsx(Tooltip, {
          content: /*#__PURE__*/_jsx(CustomTooltip, {
            external: {
              linesConfig: lines,
              xformatter: (xAxisOptions === null || xAxisOptions === void 0 ? void 0 : xAxisOptions.tooltipFormatter) || (xAxisOptions === null || xAxisOptions === void 0 ? void 0 : xAxisOptions.formatter),
              leftUnit: leftYAxisOptions === null || leftYAxisOptions === void 0 ? void 0 : leftYAxisOptions.unit,
              rightUnit: rightYAxisOptions === null || rightYAxisOptions === void 0 ? void 0 : rightYAxisOptions.unit,
              showInactives: tooltip === null || tooltip === void 0 ? void 0 : tooltip.showInnactives
            }
          })
        }), !(legend !== null && legend !== void 0 && legend.hide) && /*#__PURE__*/_jsx(Legend, {
          verticalAlign: (legend === null || legend === void 0 ? void 0 : legend.verticalAlign) || 'bottom',
          content: /*#__PURE__*/_jsx(CustomLegend, {
            external: {
              linesConfig: lines,
              align: (legend === null || legend === void 0 ? void 0 : legend.horizontalAlign) || 'right',
              showInactives: legend === null || legend === void 0 ? void 0 : legend.showInactives,
              isRightAxisDisplayed: (rightYAxisOptions === null || rightYAxisOptions === void 0 ? void 0 : rightYAxisOptions.hide) === false
            },
            selection: selectedLines,
            hasLineSelection: hasLineSelection,
            onLegendClicked: onLegendClicked,
            onLegendHovered: onLegendHovered
          })
        }), lines.map(options => /*#__PURE__*/_jsx(Line, {
          id: `line_${options.key}`,
          yAxisId: options.axis || 'left',
          dataKey: options.key,
          stroke: options.color,
          type: "monotone",
          strokeDasharray: options !== null && options !== void 0 && options.dashed ? '17 4' : '',
          connectNulls: true,
          animationDuration: 300,
          ...getLineStyleFromStatus((options === null || options === void 0 ? void 0 : options.status) || 'active', options.key),
          onClick: () => onLineClicked(options.key),
          onMouseEnter: () => onLineHovered(options.key),
          onMouseLeave: () => onLineHovered('')
        }, options.key))]
      })
    })
  });
}
export default LineChart;
//# sourceMappingURL=LineChart.component.js.map